@@ -1,15 +1,16 @@
 ##' @S3method print summary.biprobit
 print.summary.biprobit <- function(x,digits = max(3, getOption("digits") - 2),...) {
   cat("\n")
-  print(x$object,digits=digits)
-  S <- colSums(x$object$score);  names(S) <- rep("",length(S))
+  printCoefmat(x$coef,digits=digits,...)
+##  print(x$object,digits=digits)
+  S <- colSums(x$score);  names(S) <- rep("",length(S))
   cat("\n")
-  print(x$object$N,quote=FALSE)
+  print(x$N,quote=FALSE)
   ##  suppressMessages(browser())
   cat("Score: "); cat(formatC(S,...));
-  cat("\nlogLik: "); cat(sum(x$object$logLik),"\n");
-  if (!is.null(x$object$msg)) {
-    cat(x$object$msg,"\n")
+  cat("\nlogLik: "); cat(sum(x$logLik),"\n");
+  if (!is.null(x$msg)) {
+    cat(x$msg,"\n")
   }
 
   if (!is.null(x$varcomp)) {
